#define DEBUG

#include "tmdlib/TMDlib.h"

#include <iostream>
#include <sstream>
#include <string>  
#include <fstream>
using namespace std;
using namespace TMDlib;
/* Common Block Declarations */

extern string pdfpath;


 void TMD::ccfm_gluon(double x, double kt, double p, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm, double& cbar, double& bottom, double& bbar, double& top, double& tbar, double& phot,  double& glu)
{
/** 
* Routine to interpolate TMDgrids and to return momentum weighted TMD parton density as function of x, kt, scale p for different flavors. 
*
* General interpolation routine for PB TMDs (as well as CCFM uPDFs). 
*
* TMDgrid in form of 51,51,51 for logx, logkt2 and log p, for flavours [-6,..,0,... 6, 22], note id=0->id=21 gluon
*
* Parametrization for protons. 
* 
*/
    /* Initialized data */

    static int na[3] = { 51,51,51 };



    /* Local variables */
    int irr, i, j, k, in, ip;
    int igrid;
    char txt[72], adum[1];
    double rp,rx,rq2;
    double rkms0, rkms1, rkms2, rkms3, rkms4, rkms5, rkms6, rkms7, rkms1b, rkms2b, rkms3b, rkms4b, rkms5b, rkms6b;
    double bb, ktqt, qcdlam, scal, qtd1, ktd1;
    double uval,dval;

    string filename;
    
    cout << " in ccfm_gluon " << first << endl;

    /* Function Body */
    if (first) {
	n1min = 0;
	n1max = 0;
	n2min = 0;
	n2max = 0;
	n3min = 0;
	n3max = 0;
	ncall = 0;
	i = 0;
	scal = 1.;
	ipgg = 0;
      rkms0 =0;  rkms1 =0; rkms2=0; rkms3=0; rkms4=0; rkms5=0; rkms6=0; rkms7=0; 
      rkms1b=0;  rkms2b=0; rkms3b=0; rkms4b=0; rkms5b=0; rkms6b=0;
	ns_sel = -1;

      cout << " ccfm_gluon: name = " << TMD_Name << " NewFormatDir " << TMD_Dir << " with member " << TMD_Mem << endl;
  
      filename=pdfpath+TMD_Dir+TMD_Name+"_"+TMD_Mem+".dat"; 

      string text1,text2,text3,text4,textdum;
      const char *testname = filename.c_str();
#ifdef DEBUG
	cout <<  "ccfm_gluon: read grid from " << filename << endl;
      cout << testname << endl;
#endif      
      ifstream myfile (testname);
      if (myfile.is_open()) {

	   for (j = 1; j <= 51; ++j) {
	      for (i = 1; i <= 51; ++i) {
		   for (k = 1; k <= 51; ++k) {
                   if (!myfile.eof() ) {
                        myfile >>  rx >> rq2 >>rp >> rkms4b >>rkms3b >> rkms2b >> rkms1b >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4; 
                        //cout << " rx " << rx << " " << rq2 << " " << rp << " rkms0 " << rkms0  <<  endl;
                        //cout << " at  " << j << " " << i << " " << k << endl;
                   } 
               if (rx != rx) rx = 0.;
               if (rq2 != rq2) rq2 = 0.;
               if (rp != rp) rp = 0.;
               if (rkms0 != rkms0) rkms0=0;
		   xx[i] = rx;
		   q2x[j] = rq2;
		   px[k] = rp;
               // cout << i << " " << j <<" " << q2x[j] << " " << k << endl;
		    f_grid0[j + (i + k * 51) * 51 - 2653] = max(0.,rkms0) * scal;
		   }
	      }
	   }
        //  cout << " end of file: ccfm_gluon at " << j << " " << i << " " << k << endl;
        //  cout << " params in ccfm_gluon x  limits " << xx[0]<< " " <<xx[1] <<" " << xx[2] << " " << xx[51] <<  endl;
        //  cout << " params in ccfm_gluon kt2  limits " << q2x[0] << " "<< q2x[1] << " "<< q2x[2] << " " << q2x[51] <<  endl;
        //  cout << " params in ccfm_gluon p  limits " << px[0] << " "<< px[1] << " "<< px[2] << " " << px[51] <<  endl;
      }
      else { cout << " error file not found " << endl;
         return ; 
      }
	in = 0;
	for (i = 1; i <= 51; ++i) {
	    ++in;
	    a[in - 1] = q2x[i];
	}
	for (i = 1; i <= 51; ++i) {
	    ++in;
	    a[in - 1] = xx[i];
	}
	for (i = 1; i <= 51; ++i) {
	    ++in;
	    a[in - 1] = px[i];
	}

	first = false;
#ifdef DEBUG
	cout <<  "  parton densities read from file unit 30 " << endl;
#endif
      //cout <<  " end init:  iset = " << iset << " f_grid0[500] " << f_grid0[9 + (5 + 10 * 51) * 51 - 2653] << endl;
    }
    // initialise values to zero
    	glu = 0; up = 0; ubar  = 0 ; uval = 0 ; dn = 0; dbar = 0; dval = 0;
      strange = 0; sbar = 0; charm = 0; cbar = 0; bottom = 0; bbar = 0;
	top = 0; tbar = 0; phot = 0;
      
      
     //  cout <<  " in loop:  iset = " << iset << endl;

    
    ++ncall;
    q2 = kt*kt ;
    // cout << " params in ccfm_gluon: " <<x  << " " << kt << " " <<  p << endl;

    xa[0] = log(q2);
    xa[1] = log(x);
    xa[2] = log(p);
    //cout << " ccfm_gluon = " << x << " " << q2 << " " << p << endl;
    //cout << " params in ccfm_gluon: " <<xa[0] << " " << xa[1] << " " <<  xa[2] << endl;
    //cout << " params in ccfm_gluon x: " << xa[1] << " limits " << xx[1] << " " << xx[51] <<  endl;
    //cout << " params in ccfm_gluon kt2: " << xa[0] << " limits " << q2x[1] << " " << q2x[51] <<  endl;
    //cout << " params in ccfm_gluon p: " << xa[2] << " limits " << px[1] << " " << px[51] <<  endl;
    if (xa[2] < px[1]) {
	++n3min;
	if (n3min < 10) {
           cout <<" TMDlib:ccfm_gluon p out of range " << p << " min p " << exp(px[1]) <<endl;
           // cout <<" TMDlib:ccfm_gluon p out of range " << xa[2] << " " << px[1] << endl;
	} else if (n3min == 10) {
           cout << " TMDlib:ccfm_gluon p out of range " << p << " min p " << exp(px[1]) << endl;
           cout << " TMDlib:ccfm_gluon last message printed: min p " << endl;
	}
	xa[2] = px[1];
    }
    if (xa[2] > px[51]) {
	++n3max;
	if (n3max < 10) {
        //    write(6,*) ' p out of range ',p,' max p ',exp(px(n3)) */
	} else if (n3max == 10) {
        //    write(6,*) ' p out of range ',p,' max p ',exp(px(n3)) */
        //    write(6,*) ' last message printed: max p' */
	}
	xa[2] = px[51];
    }
    if (xa[1] >= xx[51]) {
	xa[1] = xx[51] - 1e-4;
    }
    if (xa[1] < xx[1]) {
	++n2min;
	if (n2min < 10) {
         //   write(6,*) '  x out of range ',x,' min ',exp(xx(1)) */
	} else if (n2min == 10) {
          //  write(6,*) '  x out of range ',x,' min ',exp(xx(1)) */
         //   write(6,*) ' last message printed: min x' */
	}
	xa[1] = xx[1];
    }
//    cout << " q2 " << q2 << " kt " << kt << " " << exp(q2x[1]) << endl;
    if (xa[0] < q2x[1]) {
	++n1min;
/*     	
     if (n1min < 10) {
         cout << "  k2 out of range " << q2 << "  min " << exp(q2x[1]) << endl;
	} else if (n1min == 10) {
         cout << "  k2 out of range " << q2 << "  min " << exp(q2x[1]) << endl;
         cout << " last message printed: min k2 " << endl;
	}
	//xa[0] = q2x[1];
      //q2 = exp(xa[0]);
*/      
      //cout << "  k2 out of range " << exp(xa[0]) << "  min " << exp(q2x[1]) << endl;
      return ;
    }
    if (xa[0] >= q2x[51]) {
	++n1max;
	/*
      if (n1max < 10) {
         cout << " k2 out of range " << q2 << "  max " << exp(q2x[51]) << endl;
	} else if (n1max == 10) {
         cout << " k2 out of range " << q2 << " max " <<exp(q2x[51])  << endl;
         cout << " last message printed: max k2 " <<endl;
	}
	*/
      //cout << " k2 out of range " << q2 << " max " <<exp(q2x[51])  << endl;
      //xa[0] = q2x[51] - .1;
      return ;
      //q2 = exp(xa[0]);
    }
    //cout << " after params in ccmfupf x: " << xa[1] << " limits " << xx[1] << " " << xx[51] <<  endl;
    //cout << " after params in ccmfupf kt2: " << xa[0] << " limits " << q2x[1] << " " << q2x[51] <<  endl;
    //cout << " after params in ccmfupf p: " << xa[2] << " limits " << px[1] << " " << px[51] <<  endl;
/* check if interpolation or grid wanted */
    igrid = 0;
    if (igrid == 1) {
	if (xa[0] < q2x[1]) {
	   cout << "  k2 out of range " << q2 <<  " min " << exp(q2x[1]) << " " << q2x[1] <<" "<< q2x[51] << endl;
	    xa[0] = q2x[1];
	}
	if (xa[0] >= q2x[51]) {
	    cout<<  "  k2 out of range " << q2 <<  " max " << exp(q2x[51]) << endl;
	    xa[0] = q2x[51] - .1;
	}
	i = 0;
L20:
	++i;
	if (xa[0] > a[na[0] - 1]) {
/*       write(6,*) ' q2  ',xa(1),a(na(1)) */
	    cout << " q2 not found " << q2 << endl; 
	    i = na[0];
	} else {
	    if (xa[0] >= a[i - 1] && xa[0] < a[i]) {
	    } else {
		if (i <= na[0]) {
		    goto L20;
		} else {
		    cout <<  " at 20: q2 not found " << q2 << endl;
		}
	    }
	}
	j = 0;
L30:
	++j;
	if (xa[1] >= a[na[0] + j - 1] && xa[1] < a[na[0] + j]) {
	} else {
	    if (j <= na[1]) {
		goto L30;
	    } else {
		cout << " at 30: x not found " << x << endl;
	    }
	}
	k = 0;
L40:
	++k;
	if (xa[2] >= a[na[0] + na[1] + na[2] - 1]) {
	    k = na[2];
	} else {
	    if (xa[2] >= a[na[0] + na[1] + k - 1] && xa[2] < a[na[0] + na[1] 
		    + k]) {
	    } else {
		if (k <= na[2]) {
		    goto L40;
		} else {
		    cout << " at 40: p not found " << p << endl;
		}
	    }
	}
	glu = f_grid0[i + (j + k * 51) * 51 - 2653] / q2;
      // cout << " ccfm_gluon i j k "<< i << " " << j << " " << k << " " << glu << " " << q2 << " " << x << " " << p << endl;

    } else { 
      int c3 = 3;
      // cout << " xa values  " << exp(xa[0]) << " " << exp(xa[1]) << " " << exp(xa[2]) << endl;
	glu   = Cdhfint(c3, xa, na, a, f_grid0) / q2;

    }
    
    // cout << " ccfm_gluon: iset = " << iset << " gluon " << glu<< endl;

    
   cout << " ccfm_gluon: gluon = " << glu << " " << q2 << endl;
    //cout << " ccfm_gluon x,q2,p,xpq " << x<< " " << q2 << " " << p << " "<< glu<< endl;
    if (glu > 1e6) {
	cout << " ccfm_gluon x,q2,p,xpq" << x<< " " << q2 << " " << p << " "<< glu<< endl;
	cout << " i,j,k " << i << " " << j << " " << k << endl;
    }
    return ;
} 

